#!/usr/bin/python
#VulnTrack by Ogma
#License: GPL
#            .__       .___.__                     
#  __________ |  |    __| _/|__| ______________  ___
# /  ___/  _ \|  |   / __ | |  |/ __ \_  __ \  \/  /
# \___ (  <_> )  |__/ /_/ | |  \  ___/|  | \/>    < 
#/____  >____/|____/\____ | |__|\___  >__|  /__/\_ \
#     \/                 \/         \/            \/
#				www.soldierx.com
########################################################################
#	VulnTrack database class
########################################################################

import sqlite3
import os

class db(object):
	dbfile = "vulns.db"

	# Connect to databse, if it doesn't exist create it
	def __init__(self):
		if not os.path.exists(self.dbfile):
			print "Creating database"
			self.conn = sqlite3.connect(self.dbfile)
			self.conn.execute("""
			create table vulns (
				id					integer primary key autoincrement not null,
				cve				text,
				description		text not null,
				date				text not null,
				terms			text not null,
				difficulty		text,
				vector			text,
				score			text,
				exploit			text,
				constraint cve_unique unique(cve));""")
			self.conn.execute("""
			create table groups (
				id					integer primary key autoincrement not null,
				name				text not null,
				constraint group_unique unique(name));""")
			self.conn.execute("""
			create table vulns_has_groups (
				vulns_id			integer not null,
				groups_id			integer not null,
				alert				integer not null,
				acknowledged		integer not null,
				primary key(vulns_id, groups_id));""")
			self.conn.execute("""
			create table exploit (
				edbid		integer not null primary key,
				cve			text not null,
				date		text not null,
				platform	text not null,
				description text not null,
				constraint group_unique unique(edbid));""")
			self.conn.execute("""
			create table versions (
				vulns_id			integer not null,
				version			text not null,
				primary key(vulns_id,version));""")
			
			self.conn.execute("""
			create table vuln_reference (
				vulns_id		integer not null,
				url			text not null,
				primary key(vulns_id,url));""")
			self.conn.commit()
		else:
			self.conn = sqlite3.connect(self.dbfile)
	
	# Insert vulnerability entry into database
	def insert(self, vulnitem, group, searchterms):
		cursor = self.conn.cursor()
		# Ghetto formatting for Murican dates
		date = vulnitem[2].split('T')[0].split('-')
		date = date[1] + "-" + date[2] + "-" + date[0]
		cursor.execute("""
		insert or ignore into vulns(cve, description, date, terms, difficulty, vector, score) 
			values( :cve, :description, :date, :terms, :difficulty, :vector, :score);""", 
		[vulnitem[0], vulnitem[1], date, searchterms, vulnitem[4], vulnitem[5], vulnitem[3]])
		# Retrieve vuln id  from the vulns table
		cursor.execute("""select id from vulns where cve = :cve;""", (vulnitem[0], ))
		vuln_id = cursor.fetchone()[0]
		
		# Check to see if group already exists in database
		cursor.execute("""select * from groups where name = :name;""", (group,))
		group_entry = cursor.fetchone()
		if group_entry:
			group_id = group_entry[0]
		else:
			cursor.execute("""insert into groups(name) values( :group );""", (group,))
			group_id = cursor.lastrowid
		# populate the vulns has group table
		cursor.execute("""insert or ignore into vulns_has_groups(vulns_id, groups_id, alert, acknowledged) values(:vuln, :group, 0, 0);""",(vuln_id,group_id))
		# insert versions
		for v in vulnitem[6]:
			cursor.execute("""insert or ignore into versions(vulns_id, version) values(:vuln, :version);""", (vuln_id, v))
		# insert references
		for r in vulnitem[7]:
			print "Inserting reference: ", vuln_id, r
			cursor.execute("""insert or ignore into vuln_reference(vulns_id, url) values(:vuln, :url);""", (vuln_id, r))
		self.conn.commit()
	
	# insert exploit into db
	def insertexploit(self, edbid, cve, date, platform, description):
		cursor = self.conn.cursor()
		cursor.execute("""insert or ignore into exploit(edbid, cve, date, platform, description) values(:edbid, :cve, :date, :platform, :description);""",
		(edbid, cve, date, platform, description))
		self.conn.commit()
		
	# exploit count
	def exploitcount(self):
		cursor = self.conn.cursor()
		cursor.execute("""select count(*) from exploit;""")
		return cursor.fetchall()[0][0]
	
	# Retrieve all exploits
	def selectexploits(self):
		cursor = self.conn.cursor()
		cursor.execute("""select * from exploit;""")
		return cursor.fetchall()
	
	def recentexploits(self, last_edbid):
		cursor = self.conn.cursor()
		cursor.execute("""select * from exploit where edbid > :last_edbid and cve = '';""", (last_edbid,))
		return cursor.fetchall()
	
	def lastexploit(self):
		cursor = self.conn.cursor()
		cursor.execute("""select edbid from  exploit order by edbid desc limit 1;""")
		return cursor.fetchall()
	
	def checkforexploit(self, cve):
		cursor = self.conn.cursor()
		cursor.execute("""select vulns.cve, exploit.edbid, exploit.description, exploit.platform from vulns,exploit where vulns.cve = :cve and exploit.cve = :cve""", (cve,))
		return cursor.fetchall()
	
	# Save changes to database
	def commit(self):
		self.conn.commit()

	# Retrieve information 
	def selectvulns(self):
		cursor = self.conn.cursor() 
		cursor.execute("""select vulns.*, groups.name as group_name from vulns, vulns_has_groups, groups where
						vulns.id = vulns_has_groups.vulns_id and groups.id = vulns_has_groups.groups_id and 
						vulns_has_groups.acknowledged = 0 order by date desc;""")
		return cursor.fetchall()
		
	def selectrefs(self, item):
		cursor = self.conn.cursor()
		cursor.execute("""select * from vuln_reference where vuln_reference.vulns_id= :id""",(item,))
		return cursor.fetchall() 
	
	def selectver(self, item):
		cursor = self.conn.cursor()
		cursor.execute("""select * from versions where versions.vulns_id = :id""", (item,))
		return cursor.fetchall() 
	
	# Get the number of vulnerabilities that haven't been alerted yet
	def count(self):
		cursor = self.conn.cursor()
		cursor.execute("""select count(*) from vulns, vulns_has_groups, groups
						where vulns.id = vulns_has_groups.vulns_id
						and groups.id = vulns_has_groups.groups_id
						and vulns_has_groups.alert = 0""")
		return cursor.fetchone()
	
	# Remove a vulnerability from the database
	def remove(self, item, group=None):
		if group is None: # Remove all instances of the vulnerability
			self.conn.execute("""delete from vulns where id= :id""", (item,))
			self.conn.execute("""delete from vulns_has_groups where vulns_id= :id""", (item,))
		else: # only remove the CVE from that group
			self.conn.execute("""delete from vulns_has_groups where
			vulns_id= :id and 
			vulns_has_groups.groups_id = (select id from groups where groups.name = :group)""", (item, group));
		self.conn.commit()
	
	# Acknowledge a vulnerability so it won't show up in the window anymore
	# This functionality will be expanded upon later to make it more useful
	def acknowledge(self, item, group=None):
		if group is None:
			self.conn.execute("""update vulns_has_groups set acknowledged=1 where vulns_id= :id""",(item,))
		else:
			self.conn.execute("""update vulns_has_groups set acknowledged=1 where
			vulns_id= :id and
			vulns_has_groups.groups_id = (select id from groups where groups.name = :group)""", (item, group));
		self.conn.commit()
	
	def togglealert(self):
		self.conn.execute("update vulns_has_groups set alert=1 where alert=0")
		self.conn.commit()

